# Generated by Django 2.2.3 on 2019-07-02 18:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('refs', '0002_auto_20190702_1258'),
    ]

    operations = [
        migrations.AddField(
            model_name='type_param',
            name='pick_list',
            field=models.TextField(blank=True, null=True),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-09 18:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0044_auto_20200209_1741'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='progresses',
            name='guid',
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-21 01:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0005_auto_20190821_0056'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='Calendar_shits_days',
            new_name='Calendar_shifts_days',
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-23 19:31

from django.db import migrations
import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0041_shift_day_relax'),
    ]

    operations = [
        migrations.AddField(
            model_name='shift_day_relax',
            name='code',
            field=isc_common.fields.code_field.CodeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='shift_day_relax',
            name='description',
            field=isc_common.fields.description_field.DescriptionField(),
        ),
        migrations.AddField(
            model_name='shift_day_relax',
            name='name',
            field=isc_common.fields.name_field.NameField(blank=True, null=True),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-09 17:28

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0042_progresses_type'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='progresses',
            name='showCancel',
        ),
        migrations.AddField(
            model_name='progresses',
            name='props',
            field=bitfield.models.BitField((('showCancel', 'Показать кнопку отменить процесс'),), db_index=True, default=0),
        ),
    ]



from django.urls import path

from tracker.views import messages_files_refs

urlpatterns = [

    path('Messages_files_refs/Fetch/', messages_files_refs.Messages_files_refs_Fetch),
    path('Messages_files_refs/Add', messages_files_refs.Messages_files_refs_Add),
    path('Messages_files_refs/Update', messages_files_refs.Messages_files_refs_Update),
    path('Messages_files_refs/Remove', messages_files_refs.Messages_files_refs_Remove),
    path('Messages_files_refs/Lookup/', messages_files_refs.Messages_files_refs_Lookup),
    path('Messages_files_refs/Info/', messages_files_refs.Messages_files_refs_Info),
    path('Messages_files_refs/Copy', messages_files_refs.Messages_files_refs_Copy),

]



# Generated by Django 3.0.7 on 2020-06-30 13:10

from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('reports', '0009_auto_20200612_2242'),
    ]

    operations = [
        migrations.AddField(
            model_name='jasper_reports',
            name='report_id',
            field=isc_common.fields.code_field.CodeStrictField(null=True),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-09 15:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0039_progresses_guid'),
    ]

    operations = [
        migrations.AlterField(
            model_name='progresses',
            name='guid',
            field=models.UUIDField(),
        ),
    ]



import hashlib
import logging
import uuid

from django.conf import settings
from django.db import transaction
from django.db.models import UUIDField, CharField, UniqueConstraint, Q
from django.forms import model_to_dict

from isc_common import setAttr, delAttr
from isc_common.auth.http.LoginRequets import LoginRequest
from isc_common.auth.models.user import User
from isc_common.bit import IsBitOn
from isc_common.fields.description_field import DescriptionField
from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_tree_grid_manager import CommonTreeGridManager
from isc_common.models.audit import AuditQuerySet
from isc_common.models.base_ref import Hierarcy
from isc_common.number import DelProps
from tracker.models.messages_importance import Messages_importance
from tracker.models.messages_state import Messages_state
from tracker.models.messages_theme import Messages_theme

logger = logging.getLogger(__name__)


class MessagesQuerySet(AuditQuerySet):

    def create(self, **kwargs):
        setAttr(kwargs, 'checksum', hashlib.md5(kwargs.get('message').encode()).hexdigest())
        setAttr(kwargs, 'message', f'<pre>{kwargs.get("message")}</pre>')
        return super().create(**kwargs)

    def update(self, **kwargs):
        setAttr(kwargs, 'checksum', hashlib.md5(kwargs.get('message').encode()).hexdigest())
        setAttr(kwargs, 'message', f'<pre>{kwargs.get("message")}</pre>')
        return super().update(**kwargs)

    def delete(self):
        from tracker.models.messages_files_refs import Messages_files_refs
        from tracker.models.messages_files import Messages_files

        with transaction.atomic():
            for message in self:
                for messages_files_refs in Messages_files_refs.objects.filter(message_id=message.id):
                    if Messages_files_refs.objects.filter(messages_file=messages_files_refs.messages_file).count() == 1:
                        id = messages_files_refs.messages_file.id
                        messages_files_refs.delete()
                        Messages_files.objects.filter(id=id).delete()
                    else:
                        messages_files_refs.delete()

                return super().delete()


class MessagesManager(CommonTreeGridManager):

    @staticmethod
    def get_tracker_bot():
        key='MessagesManager.get_tracker_bot'
        settings.LOCKS.acquire(key)
        tracker_bot, _ = User.objects.get_or_create(username='tracker_bot', last_name='Бот (Дела)', password='1234567890', props=User.props.bot, deliting=False, editing=False)
        settings.LOCKS.release(key)
        return tracker_bot

    @staticmethod
    def getRecord(record):
        res = {
            'date_create': record.lastmodified,
            'deliting': record.deliting,
            'editing': record.editing,
            'enabled': record.state.code != 'closed',
            'guid': str(record.guid).upper(),
            'id': record.id,
            'importance__code': record.importance.code if record.importance else None,
            'importance__name': record.importance.name if record.importance else None,
            'importance_id': record.importance.id if record.importance else None,
            'lastmodified': record.lastmodified,
            'message': record.message,
            'parent_id': record.parent_id,
            'state__name': record.state.name,
            'state_id': record.state.id,
            'theme__full_name': record.theme.full_name,
            'theme__name': record.theme.name,
            'theme_id': record.theme.id,
            'to_whom__short_name': record.to_whom.get_short_name,
            'to_whom__username': record.to_whom.username,
            'to_whom_id': record.to_whom.id,
            'user__color': record.user.color if record.user.color != None and record.user.color != 'undefined' else 'black',
            'user__full_name': record.user.get_full_name,
            'user__short_name': record.user.get_short_name,
            'user_id': record.user.id if record.user else None,
        }
        return DelProps(res)

    def get_queryset(self):
        return MessagesQuerySet(self.model, using=self._db)

    def createAutoErrorFromRequest(self, request, printRequest=False, function=None):
        request = DSRequest(request=request)
        data = request.get_data()
        setAttr(data, 'state', Messages_state.objects.get(code='new'))
        setAttr(data, 'theme', Messages_theme.objects.get(code='auto_from_error'))
        setAttr(data, 'to_whom', User.objects.get(username='developer'))
        message = data.get('message', None)
        user_id = data.get('user_id', None)
        setAttr(data, 'user', User.objects.get(id=user_id))

        if message and isinstance(message, list):
            message = '\n'.join(message)
            setAttr(data, 'message', message)
        return super().create(**data)

    @staticmethod
    def get_border(message, user=None):
        if isinstance(user, int):
            color = User.objects.get(id=user).color
            return f'<div style="border: 1px solid {color if color else "black"};" class="outline">{message}</div>'
        elif isinstance(user, User):
            return f'<div style="border: 1px solid {user.color if user.color else "black"};" class="outline">{message}</div>'
        else:
            return f'<div style="border: 1px solid "black";" class="outline">{message}</div>'

    def createFromRequest(self, request, printRequest=False, function=None):
        from tracker.models.messages_files_refs import Messages_files_refs

        request = DSRequest(request=request)
        data = request.get_data()
        data_clone = data.copy()
        messageFileIds = data_clone.get('messageFileIds')
        delAttr(data_clone, 'messageFileIds')

        delAttr(data_clone, 'user__username')
        delAttr(data_clone, 'state__name')
        delAttr(data_clone, 'theme__full_name')
        delAttr(data_clone, 'theme__name')
        delAttr(data_clone, 'importance__name')
        delAttr(data_clone, 'isFolder')
        delAttr(data_clone, 'to_whom__username')
        delAttr(data_clone, 'user__short_name')
        delAttr(data_clone, 'to_whom__short_name')

        to_whom = data_clone.get('to_whom')
        messages = []
        with transaction.atomic():
            if isinstance(to_whom, list):
                delAttr(data_clone, 'to_whom')
                setAttr(data_clone, 'lastmodified', data_clone.get('date_create'))
                delAttr(data_clone, 'date_create')

                for to_whom_id in to_whom:
                    setAttr(data_clone, 'to_whom_id', to_whom_id)
                    setAttr(data_clone, 'guid', uuid.uuid4())
                    res = super().create(**data_clone)

                    if isinstance(messageFileIds, list):
                        for messageFileId in messageFileIds:
                            Messages_files_refs.objects.get_or_create(message=res, messages_file_id=messageFileId)

                    if request.user_id != to_whom_id:
                        LoginRequest.send_bot_message(user=to_whom_id,
                                                      bot=MessagesManager.get_tracker_bot(),
                                                      message=f'<h3>Вам назначено новое дело:</h3> #{res.guid}'
                                                              f'<p>Тема: {res.theme.full_name}'
                                                              f'<p>{MessagesManager.get_border(res.message, res.user)}'
                                                              f'<p/>Статус: "{res.state.name}"'
                                                              f'<p/>Важность: "{res.importance.name}"'
                                                              f'<p/>Назначил: {res.user.get_short_name}'
                                                      )
                    messages.append(model_to_dict(res))
            else:
                setAttr(data_clone, 'guid', uuid.uuid4())
                res = super().create(**data_clone)

                if isinstance(messageFileIds, list):
                    for messageFileId in messageFileIds:
                        Messages_files_refs.objects.get_or_create(message=res, messages_file_id=messageFileId)

                if request.user_id != res.to_whom.id:
                    LoginRequest.send_bot_message(user=res.to_whom, bot=MessagesManager.get_tracker_bot(),
                                                  message=f'<h3>Вам назначено новое дело:</h3> #{res.guid}'
                                                          f'<p/>Тема: {res.theme.full_name}'
                                                          f'<p/>{MessagesManager.get_border(res.message, res.user)}'
                                                          f'<p/>Статус: "{res.state.name}"'
                                                          f'<p/>Важность: "{res.importance.name}"'
                                                          f'<p/>Назначил: {res.user.get_short_name}'
                                                  )
                messages.append(model_to_dict(res))

        return messages

    def close_rescurce(self, id, request):
        state_id = Messages_state.message_state_closed().id
        state__name = Messages_state.message_state_closed().name

        if not request.is_admin and not request.is_develop and Messages.objects.get(id=id).user.id != request.user_id:
            raise Exception(f'Закрытие может выполнить только автор.')

        for message in Messages.objects.filter(parent_id=id):
            message.state_id = state_id
            message.save()

            if request.user_id != message.to_whom.id:
                user = User.objects.get(id=request.user_id)
                LoginRequest.send_bot_message(user=message.to_whom.id, bot=MessagesManager.get_tracker_bot(),
                                              message=f'<h3>Изменение дела:</h3> #{message.guid}'
                                                      f'<p/>Тема: {message.theme.full_name}'
                                                      f'<p/>{MessagesManager.get_border(message.message, user)}'
                                                      f'<p/>Статус: "{state__name}"'
                                                      f'<p/>Важность: "{message.importance.name}"'
                                                      f'<p/>Изменил: {user.get_short_name}'
                                              )
            self.close_rescurce(message.id, request)

    def updateFromRequest(self, request):
        from tracker.models.messages_files_refs import Messages_files_refs

        request = DSRequest(request=request)
        data = request.get_data()
        old_data = request.get_oldValues()

        data_clone = data.copy()
        messageFileIds = data_clone.get('messageFileIds')
        delAttr(data_clone, 'messageFileIds')
        delAttr(data_clone, 'user__username')
        delAttr(data_clone, 'user__short_name')
        delAttr(data_clone, 'to_whom__short_name')

        delAttr(data_clone, 'state__name')

        theme__full_name = data_clone.get('theme__full_name')
        delAttr(data_clone, 'theme__full_name')
        delAttr(data_clone, 'theme__name')
        delAttr(data_clone, 'importance__name')
        delAttr(data_clone, 'importance__code')
        delAttr(data_clone, 'enabled')

        delAttr(data_clone, 'id')
        delAttr(data_clone, 'isFolder')
        delAttr(data_clone, 'parent')
        id = request.get_id()

        old_to_whom_id = old_data.get('to_whom_id')
        to_whom_id = data.get('to_whom_id')

        if old_to_whom_id != to_whom_id:
            setAttr(data_clone, 'user_id', request.user_id)
            setAttr(data_clone, 'parent_id', id)
            setAttr(data_clone, 'guid', uuid.uuid4())
            setAttr(data, 'message', old_data.get('message'))

            with transaction.atomic():
                res = super().create(**data_clone)

                if isinstance(messageFileIds, list):
                    for messageFileId in messageFileIds:
                        Messages_files_refs.objects.get_or_create(message=res, messages_file_id=messageFileId)

                LoginRequest.send_bot_message(user=to_whom_id,
                                              bot=MessagesManager.get_tracker_bot(),
                                              message=f'<h3>Вам назначено новое дело:</h3>#{data_clone.get("guid")}'
                                                      f'<p/>Тема: {theme__full_name}'
                                                      f'<p/>{MessagesManager.get_border(data_clone.get("message"), res.user)}'
                                                      f'<p/>Статус: "{Messages_state.objects.get(id=data_clone.get("state_id")).name}"'
                                                      f'<p/>Важность: "{Messages_importance.objects.get(id=data_clone.get("importance_id")).name}"'
                                                      f'<p/>Назначил: {res.user.get_short_name}'
                                              )
        else:
            with transaction.atomic():
                res = super().filter(id=id).update(
                    importance_id=data_clone.get('importance_id'),
                    message=data_clone.get('message'),
                    state_id=data_clone.get('state_id'),
                    to_whom_id=data_clone.get('to_whom_id'),
                )

                if isinstance(messageFileIds, list):
                    for message in Messages.objects.filter(id=id):
                        for messageFileId in messageFileIds:
                            Messages_files_refs.objects.get_or_create(message=message, messages_file_id=messageFileId)

                if res > 0:
                    user_id = None
                    if request.user_id != data_clone.get('to_whom_id'):
                        user_id = data_clone.get('to_whom_id')

                    if request.user_id != data_clone.get('user_id'):
                        user_id = data_clone.get('user_id')

                    if user_id:
                        user = User.objects.get(id=request.user_id)
                        LoginRequest.send_bot_message(user=user_id,
                                                      bot=MessagesManager.get_tracker_bot(),
                                                      message=f'<h3>Изменение дела:</h3>#{data_clone.get("guid")}'
                                                              f'<p/>Тема: {data.get("theme__full_name")}'
                                                              f'<p/>{MessagesManager.get_border(data.get("message"), user)}'
                                                              f'<p/>Cтатус: {Messages_state.objects.get(id=data_clone.get("state_id")).name}'
                                                              f'<p/>Важность: {Messages_importance.objects.get(id=data_clone.get("importance_id")).name}'
                                                              f'<p/>Изменил: {user.get_short_name}')

                    if data_clone.get('state_id') == Messages_state.message_state_closed().id:
                        setAttr(data, 'enabled', False)
                        self.close_rescurce(id, request)

        return data

    def deleteFromRequest(self, request, removed=None, ):
        from tracker.models.messages_files_refs import Messages_files_refs
        request = DSRequest(request=request)
        res = 0
        tuple_ids = request.get_tuple_ids()
        with transaction.atomic():
            for id, mode in tuple_ids:
                if mode == 'hide':
                    message = super().get(id=id)
                    if not request.is_admin and not request.is_develop and message.user.id != request.user_id:
                        raise Exception(f'Удаление может выполнить только автор.')
                    message.soft_delete()
                else:
                    message = super().get(id=id)
                    if not message.user.props.bot.is_set:
                        if not request.is_admin and not request.is_develop and message.user.id != request.user_id:
                            raise Exception(f'Удаление может выполнить только автор.')

                    Messages_files_refs.objects.filter(message=message).delete()
                    qty, _ = message.delete()

                    if request.user_id != message.user.id:
                        user = User.objects.get(id=request.user_id)
                        if not IsBitOn(message.user.props, 0):
                            LoginRequest.send_bot_message(user=message.to_whom.id,
                                                          bot=MessagesManager.get_tracker_bot(),
                                                          message=f'<h3>Удаление дела:</h3> #{message.guid}'
                                                                  f'<p/>Тема: {message.theme.full_name}'
                                                                  f'<p/>{MessagesManager.get_border(message.message, user)}'
                                                                  f'<p/>Статус: {message.state.name}'
                                                                  f'''<p/>Важность: {message.importance.name if message.importance else 'Не определена'}'''
                                                                  f'<p/>Удалил: {user.get_short_name}'
                                                          )
                    res += qty
        return res


class Messages(Hierarcy):
    checksum = CharField(max_length=255)
    guid = UUIDField(blank=True, null=True)
    importance = ForeignKeyProtect(Messages_importance, blank=True, null=True)
    message = DescriptionField(null=False, blank=False)
    state = ForeignKeyCascade(Messages_state)
    theme = ForeignKeyProtect(Messages_theme)
    to_whom = ForeignKeyCascade(User, related_name='user_msg_to_whom')
    user = ForeignKeyCascade(User, related_name='user_msg_user')

    def __str__(self):
        return f'id: {self.id}, guid: {str(self.guid).upper()}, message: {self.message}, user: [{self.user}], theme: [{self.theme}], state: [{self.state}]'

    objects = MessagesManager()

    class Meta:
        verbose_name = 'Сообщения'
        constraints = [
            UniqueConstraint(fields=['checksum', 'state', 'theme', 'to_whom', 'user'], condition=Q(guid=None), name='Messages_unique_constraint_0'),
            UniqueConstraint(fields=['checksum', 'guid', 'state', 'theme', 'to_whom', 'user'], name='Messages_unique_constraint_1'),
        ]
        ordering = ('lastmodified',)



# Generated by Django 2.2.5 on 2019-09-07 04:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0060_auto_20190907_0438'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='messages_files_refs',
            unique_together={('message', 'messages_file')},
        ),
    ]



import json

from bitfield import BitHandler
from django.core.serializers.json import DjangoJSONEncoder
from django.db.models.fields.files import FieldFile
from django.http import HttpResponse

from isc_common import setAttr


class JsonResponse(HttpResponse):
    """
    An HTTP response class that consumes data to be serialized to JSON.

    :param data: Data to be dumped into json. By default only ``dict`` objects
      are allowed to be passed due to a security flaw before EcmaScript 5. See
    :param encoder: Should be a json encoder class. Defaults to
      ``django.core.serializers.json.DjangoJSONEncoder``.
    :param safe: Controls if only ``dict`` objects may be serialized. Defaults
      to ``True``.
    :param json_dumps_params: A dictionary of kwargs passed to json.dumps().
    """

    def __init__(self, data, encoder=DjangoJSONEncoder, safe=True, json_dumps_params=None, **kwargs):
        if safe and not isinstance(data, dict):
            raise TypeError(
                'In order to allow non-dict objects to be serialized set the '
                'safe parameter to False.'
            )
        if json_dumps_params is None:
            json_dumps_params = dict()
        kwargs.setdefault('content_type', 'application/json')
        self.container_data = data
        data = json.dumps(data, cls=encoder, **json_dumps_params)
        super().__init__(content=data, **kwargs)


class JsonResponseLookup(HttpResponse):

    def __init__(self, data, encoder=DjangoJSONEncoder, safe=True, json_dumps_params=None, **kwargs):
        if safe and not isinstance(data, dict):
            raise TypeError(
                'In order to allow non-dict objects to be serialized set the '
                'safe parameter to False.'
            )
        if json_dumps_params is None:
            json_dumps_params = dict()

        kwargs.setdefault('content_type', 'application/json')
        self.container_data = data.copy()

        _data = self.container_data.get('response').get('data')
        _data_clear = dict()

        for key, value in _data.items():
            if not isinstance(value, FieldFile):
                setAttr(_data_clear, key, value)
            if isinstance(value, BitHandler):
                setAttr(_data_clear, key, value._value)

        data['response']['data'] = _data_clear

        data = json.dumps(data, cls=encoder, **json_dumps_params)
        super().__init__(content=data, **kwargs)



from files_blk.models.files_group import Files_group
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse


@JsonResponseWithException()
def Files_group_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Files_group.objects.
                select_related().
                get_range_rows1(
                request=request,
                # function=Files_groupManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Files_group_Add(request):
    return JsonResponse(DSResponseAdd(data=Files_group.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Files_group_Update(request):
    return JsonResponse(DSResponseUpdate(data=Files_group.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Files_group_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Files_group.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Files_group_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Files_group.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Files_group_Info(request):
    return JsonResponse(DSResponse(request=request, data=Files_group.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Files_group_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Files_group.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



from django.conf import settings


def make_shedurer_tasks():
    from isc_common.management.commands.grop_tmp_mat_views import grop_mat_views
    from pyriodic import Scheduler, DurationJob

    s = Scheduler()

    def drop_tbl():
        grop_mat_views(prefix='ready')
        grop_mat_views(prefix='tmp')

    s.add_job(DurationJob(drop_tbl, when=settings.SHEDULLER_PERIOD, name='grop_ready_mat_views'))

    # print(s.next_run_times())
    return s


def st_sheduler():
    try:
        if callable(settings.SHEDULLER):
            from threading import Timer
            def st():
                if callable(settings.SHEDULLER):
                    s = make_shedurer_tasks()
                    settings.SHEDULLER(s)

            t = Timer(10.0, st)
            t.start()
    except AttributeError:
        print('Sheduler not started.')
        pass


st_sheduler()



import logging

from django.conf import settings

from isc_common.models.base_ref import BaseRef, BaseRefManager, BaseRefQuerySet
from isc_common.ws.webSocket import WebSocket

logger = logging.getLogger(__name__)


class ChatsQuerySet(BaseRefQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class ChatsManager(BaseRefManager):

    @staticmethod
    def refresh_chat_menu(user, logger):
        from isc_common.auth.http.LoginRequets import LoginRequest
        message = dict(
            type='refresh_chat_menu',
            user_id=user.id,
            chatInfo=LoginRequest.get_chats(user)
        )
        WebSocket.send_message(
            host=settings.WS_HOST,
            port=settings.WS_PORT,
            channel=f'common_{user.username}',
            message=message,
            logger=logger
        )

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'description': record.description,
        }
        return res

    def get_queryset(self):
        return ChatsQuerySet(self.model, using=self._db)


class Chats(BaseRef):
    objects = ChatsManager()

    def __str__(self):
        return f"id : {self.code}, code : {self.code}, name : {self.name}, code : {self.description},"

    class Meta:
        verbose_name = 'Групповые чаты'



from typing import Text


def getOrElse(default, value):
    if value:
        return value
    else:
        return default

blue='blue'
closed='closed'
doing='doing'
green='green'
made='made'
new='new'
new_man='new_man'
red='red'
restarted='restarted'
started='started'
transferred='transferred'

def blinkString(text, blink=True, color="black", bold=False) -> Text:
    if blink:
        res = f'<div class="blink"><strong><font color="{color}"</font>{text}</strong></div>'
    else:
        res = f'<div><strong><font color="{color}"</font>{text}</strong></div>'

    if bold == True:
        return f'<b>{res}</b>'
    else:
        return res



from django.test import TestCase

from isc_common.auth.models.user import User
from tracker.models.messages import Messages
from tracker.models.messages_state import Messages_state
from tracker.models.messages_theme import Messages_theme


class Test_Messages(TestCase):

    def setUp(self):
        self.message = "message"
        self.user = User.objects.create_superuser(username="username", email="info@ivc-inform.ru", password="123")
        self.state = Messages_state.objects.get(code="new")
        self.theme = Messages_theme.objects.get(code="auto_from_error")

    def test_1(self):
        Messages_item = Messages.objects.create(message=self.message, user=self.user, state=self.state, theme=self.theme)
        Messages_item = Messages.objects.create(message=self.message, user=self.user, state=self.state, theme=self.theme)
        Messages_item = Messages.objects.get(pk=Messages_item.pk)

        self.assertEquals(Messages_item.message, self.message)
        self.assertEquals(Messages_item.user, self.user)
        self.assertEquals(Messages_item.state, self.state)
        self.assertEquals(Messages_item.theme, self.theme)
        print(f'checksum: {Messages_item.checksum}')




# Generated by Django 2.2.12 on 2020-04-28 18:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('files_blk', '0002_auto_20200428_1751'),
    ]

    operations = [
        migrations.AddField(
            model_name='files',
            name='file_size',
            field=models.PositiveIntegerField(default=None),
        ),
    ]



import logging

from django.db import transaction

from isc_common.bit import IsBitOn, TurnBitOn

logger = logging.getLogger(__name__)


class Event:
    event_type = None
    bot = None

    def __init__(self, **kwargs):
        from events.models.event_types import Event_types
        from events.models.event_type_users import Event_type_users

        for k, v in kwargs.items():
            setattr(self, k, v() if callable(v) else v)

        if not isinstance(self.event_type, Event_types):
            raise Exception(f'Not specified "event_type"')

        self.users = [event_type_users.user for event_type_users in Event_type_users.objects.filter(event_type=self.event_type)]

    def send_message(self, message=None, users_array=None, progress=None, len=None):
        from events.models.event_type_users import Event_type_usersManager
        from isc_common.auth.http.LoginRequets import LoginRequest
        from isc_common.common import blinkString
        from isc_common.progress import Progress
        from isc_common.progress import ProgressDroped
        from tracker.models.messages import MessagesManager
        from isc_common.progress import progress_deleted

        users = self.users.copy()

        if isinstance(users_array, list):
            users.extend(users_array)

        idx = 0
        if isinstance(progress, Progress) and len:
            progress.setQty(len)

        for user in users:
            if isinstance(message, list):
                _message = message[idx]
                idx += 1
            elif isinstance(message, str):
                _message = message
            else:
                raise Exception(f'Недопустимый формат message.')

            msg = f'<h3>Сообщение от:</h3> #{self.event_type.full_name}' \
                  f'<p>{MessagesManager.get_border(_message, user)}'

            if isinstance(progress, Progress):
                progress.setContentsLabel(blinkString(f'Отправка сообщения: {msg} <p/> {user.get_full_name}', bold=True, blink=False, color='blue'))

                if len:
                    if progress.step() != 0:
                        raise ProgressDroped(progress_deleted)

            LoginRequest.send_bot_message(
                user=user,
                bot=Event_type_usersManager.get_bot(self.event_type),
                message=msg,
                compulsory_reading=self.event_type.props.compulsory_reading.is_set
            )


class EventsManager:
    events = []

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v() if callable(v) else v)

    def get_event(self, code, name, compulsory_reading=True) -> Event:
        from events.models.event_types import Event_types

        events = [event for event in self.events if event.event_type.code == code]
        if len(events) == 0:
            with transaction.atomic():
                if not isinstance(code, str):
                    raise Exception(f'Not specified "code" event')

                if not isinstance(name, str):
                    raise Exception(f'Not specified "code" event')

                codes = code.split('.')
                names = name.split('.')

                if len(codes) != len(names):
                    raise Exception(f'Not mapping "code" to "name')

                code_path = None
                index = 0
                parent = None
                event = None
                for _code in codes:
                    if code_path == None:
                        code_path = f'{_code}'
                    else:
                        code_path = f'{code_path}.{_code}'

                    props = 0
                    if index == len(codes) - 1:
                        props = TurnBitOn(props, 0)
                        if compulsory_reading:
                            props = TurnBitOn(props, 1)

                    event_type, _ = Event_types.objects.update_or_create(
                        code=code_path,
                        defaults=dict(
                            name=names[index],
                            props=props,
                            parent=parent,
                            editing=False,
                            deliting=False,
                        )
                    )

                    if IsBitOn(event_type.props, 0):
                        event = Event(event_type=event_type)
                        self.events.append(event)

                    index += 1
                    parent = event_type
                return event
        else:
            return events[0]



def get_NoneStr(str):
    if str == None:
        return ''
    else:
        return str


def get_NullStr(str):
    if str == None:
        return ''
    elif str == 'null':
        return ''
    else:
        return str

def StrToBytes(mystring):
 return mystring.encode('utf-8')

def BytesToStr(mybytes):
 return mybytes.decode('utf-8')



class Node:
    def __init__(self, item=None, next=None):
        self.item = item
        self.next = next

    def __str__(self):
        # return f'{self.item}, next=[{self.next.item if self.next else None}]'
        return f'{self.item}'


class LinkedList:
    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v() if callable(v) else v)

        self.first = None
        self.last = None
        self._length = 0

    def print(self, comment=''):
        print('\n')
        print(f'======================================================= {comment} ({self.__hash__()}): =======================================================')
        print(str(self))
        print(f'===================================================== end {comment} ({self.__hash__()}) ({self.length}): =====================================================')

    def _fill_blanks(self, lenght):
        res = ''
        for i in range(lenght):
            res += ' '
        return res

    def __str__(self):
        if self.first != None:
            current = self.first

            out = f'{self.__class__.__name__} [\n{self._fill_blanks(4)}{str(current)}\n'

            while current.next != None:
                current = current.next
                out += f'{self._fill_blanks(4)}{str(current)}\n'
            return f'{out}]'

        return f'{self.__class__.__name__} []'

    def clear(self):
        self.__init__()

    @property
    def length(self):
        self._length = 0

        if self.first != None:
            self._length += 1
            current = self.first

            while current.next != None:
                current = current.next
                self._length += 1

        return self._length

    # Добавление элементов в конец списка.
    def add(self, item):
        self._length += 1
        if self.first == None:
            # self.first и self.last будут указывать на одну область памяти
            self.last = self.first = Node(item=item)
        else:
            # здесь, уже на разные, т.к. произошло присваивание
            self.last.next = self.last = Node(item=item)

    # А не научиться ли нам вставлять эл-ты в начало списка?)
    def push(self, item):
        self._length += 1
        if self.first == None:
            self.last = self.first = Node(item=item)
        else:
            self.first = Node(item=item, next=self.first)

    # Вставка в позицию
    def insert(self, pos, item):
        if self.first == None:
            self.last = self.first = Node(item=item, next=None)
            return

        if pos == 0:
            self.first = Node(item=item, next=self.first)
            return

        curr = self.first
        count = 0

        while curr != None:
            count += 1
            if count == pos:
                curr.next = Node(item=item, next=curr.next)
                if curr.next.next == None:
                    self.last = curr.next
                break
            curr = curr.next

    # Удаление головного элемента
    def pop(self):
        oldhead = self.first
        if oldhead == None:
            return None

        self.first = oldhead.next
        if self.first == None:
            self.last = None

        return oldhead.value

    # Удаление элемента из списка
    def Del(self, pos):
        if (self.first == None):
            return
        curr = self.first
        count = 0
        if pos == 0:
            self.first = self.first.next
            return
        while curr != None:
            if count == pos:
                if curr.next == None:
                    self.last = curr
                old.next = curr.next
                break
            old = curr
            curr = curr.next
            count += 1

    # Вставка в позицию
    def replace(self, pos, item):
        self.insert(pos, item)
        self.Del(pos + 1)

    # Вставка элемента в отсортированный список
    def sorted_insert(self, item):
        if self.first == None:
            self.first = Node(item, self.last)
            return
        if self.first.item > item:
            self.first = Node(item, self.first)
            return
        curr = self.first
        while curr != None:
            if curr.value > item:
                old.next = Node(item, curr)
                return
            old = curr
            curr = curr.next
        self.last = old.next = Node(item, None)

    # удаление повторяющихся значений
    def remove_duplicates(self):
        if (self.first == None):
            return
        old = curr = self.first
        while curr != None:
            _del = 0
            if curr.next != None:
                if curr.value == curr.next.value:
                    curr.next = curr.next.next
                    _del = 1
            if _del == 0:
                curr = curr.next

    def __getitem__(self, key):  # поддержка обращения по ключу
        length = 0
        current = None
        if self.first != None:
            current = self.first
            # while key != length or current.next != None:
            while key != length:
                if current.next != None:
                    current = current.next
                    length += 1
                else:
                    current = None
                    break

            if key == length:
                current = current.item

        return current

    def find(self, item, func_equal=None):
        curr = self.first
        while curr != None:
            if func_equal:
                res = func_equal(item, curr.item)
            else:
                res = curr == item

            if res:
                return curr.item
            curr = curr.next
        return None



import logging

from bitfield import BitField
from django.db import transaction, connection
from django.db.models import PositiveIntegerField
from django.forms import model_to_dict

from isc_common import setAttr
from isc_common.auth.models.user import User
from isc_common.fields.related import ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from isc_common.number import DelProps
from tracker.models.messages_theme import Messages_theme

logger = logging.getLogger(__name__)


class Chat_user_userQuerySet(AuditQuerySet):
    def create(self, **kwargs):
        from twits.models.chat_user_user_theme import Chat_user_user_theme

        user_sender = kwargs.get('user_sender')
        if not user_sender:
            user_sender = User.objects.get(id=kwargs.get('user_sender_id'))

        user_reciver = kwargs.get('user_reciver')
        if not user_reciver:
            user_reciver = User.objects.get(id=kwargs.get('user_reciver_id'))

        if user_sender.id == user_reciver.id:
            raise Exception(f'Чат с самим собою не допускается.')

        with transaction.atomic():
            try:
                common_id = self.model.objects.get(user_sender=user_reciver, user_reciver=user_sender).common_id
            except self.model.DoesNotExist:
                common_id = Chat_user_userManager.get_next_common_id()

            setAttr(kwargs, 'common_id', common_id)

            theme, created = Messages_theme.objects.get_or_create(
                code=f'tetatet_{user_sender.get_short_name}_{user_reciver.get_short_name}',
                defaults=dict(
                    name=f'Чат: {user_sender.get_short_name} : {user_reciver.get_short_name}',
                    creator=User.admin_user(),
                    editing=False,
                    deliting=False,
                    parent=Messages_theme.tetatet_chats_theme(),
                    description='Группа создана автоматически'))

            chat_user_user = super().create(**kwargs)
            Chat_user_user_theme.objects.get_or_create(theme=theme, chat_user_user=chat_user_user)
        return chat_user_user

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Chat_user_userManager(AuditManager):

    @staticmethod
    def get_next_common_id():
        with connection.cursor() as cursor:
            cursor.execute("select max(common_id) + 1 from twits_chat_user_user")
            row = cursor.fetchone()
            if not row[0]:
                return 1
            return row[0]

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'common_id': record.common_id,
            'user_sender_id': record.user_sender.id,
            'user_reciver_id': record.user_reciver.id,
            'user_reciver__username': record.user_reciver.username,
            'user_reciver__first_name': record.user_reciver.first_name,
            'user_reciver__last_name': record.user_reciver.last_name,
            'user_reciver__email': record.user_reciver.email,
            'user_reciver__middle_name': record.user_reciver.middle_name,
            'user_reciver__last_login': record.user_reciver.last_login,
            'props': record.props,
            'compulsory_reading': record.props.compulsory_reading,
        }
        return DelProps(res)

    def get_queryset(self):
        return Chat_user_userQuerySet(self.model, using=self._db)

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        user_ids = data.get('user_ids', None)
        context_ids = data.get('context_ids', None)
        _data = []

        with transaction.atomic():
            if user_ids and context_ids:
                if not isinstance(user_ids, list):
                    user_ids = [user_ids]

                if not isinstance(context_ids, list):
                    context_ids = [context_ids]

                for user_id in user_ids:
                    for user_sender_id in context_ids:
                        res, _ = super().get_or_create(user_reciver_id=user_id, user_sender_id=user_sender_id)
                        res, _ = super().get_or_create(user_reciver_id=user_sender_id, user_sender_id=user_id)

                        from twits.models.chats import ChatsManager
                        ChatsManager.refresh_chat_menu(user=res.user_reciver, logger=logger)
                        ChatsManager.refresh_chat_menu(user=res.user_sender, logger=logger)

                        _data.append(model_to_dict(res))

            return data

    def updateFromRequest(self, request, removed=None, function=None):
        from twits.models.chats import ChatsManager

        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()

        with transaction.atomic():
            props = data.get('props')
            if data.get('compulsory_reading') == True:
                props |= Chat_user_user.props.compulsory_reading
            else:
                props &= ~Chat_user_user.props.compulsory_reading
            _data = dict()
            setAttr(_data, 'props', props)

            res, _ = super().update_or_create(id=data.get('id'), defaults=_data)

            ChatsManager.refresh_chat_menu(user=res.user_reciver, logger=logger)
            ChatsManager.refresh_chat_menu(user=res.user_sender, logger=logger)

            return _data

    def deleteFromRequest(self, request, removed=None, function=None):
        request = DSRequest(request=request)
        qty = 0
        res = 0
        tuple_ids = request.get_tuple_ids()
        with transaction.atomic():
            for id, mode in tuple_ids:
                if mode == 'hide':
                    super().filter(id=id).soft_delete()
                else:
                    chat_user_user = Chat_user_user.objects.get(id=id)

                    qty, _ = super().filter(id=id).delete()

                    from twits.models.chats import ChatsManager
                    ChatsManager.refresh_chat_menu(user=chat_user_user.user_reciver, logger=logger)
                    ChatsManager.refresh_chat_menu(user=chat_user_user.user_sender, logger=logger)
                res += qty
            return res

    @staticmethod
    def props():
        return BitField(flags=(
            ('compulsory_reading', 'Обязательное прочтение'),  # 1
        ), default=0, db_index=True)


class Chat_user_user(AuditModel):
    user_sender = ForeignKeyCascade(User, related_name='user_sender_user')
    user_reciver = ForeignKeyCascade(User, related_name='user_reciver_user')
    common_id = PositiveIntegerField()
    props = Chat_user_userManager.props()

    objects = Chat_user_userManager()

    def __str__(self):
        return f'id: {self.id}, common_id: {self.common_id}, user_sender: [{self.user_sender}], user_reciver: [{self.user_reciver}]'

    class Meta:
        verbose_name = 'Чаты личной переписки'
        unique_together = (('user_sender', 'user_reciver', 'common_id'),)



import logging

from django.db.models import BooleanField

from isc_common.auth.models.user import User
from isc_common.fields.related import ForeignKeyCascade
from isc_common.managers.common_tree_grid_manager import CommonTreeGridManager
from isc_common.models.base_ref import BaseRefHierarcy

logger = logging.getLogger(__name__)


class Messages_theme_view_Manager(CommonTreeGridManager):
    @staticmethod
    def getRecord(record):
        res = {
            'id': record.get("id"),
            'code': record.get("code"),
            'name': record.get("name"),
            # 'full_name': record.get("full_name"),
            'description': record.get("description"),
            'parent_id': record.get("parent_id"),
            'lastmodified': record.get("lastmodified"),
            'isFolder': record.get("isFolder"),
            'editing': record.get("editing"),
            'deliting': record.get("deliting"),
        }
        return res

class Messages_theme_view(BaseRefHierarcy):
    user = ForeignKeyCascade(User, null=True, blank=True, related_name='user_theme')
    creator = ForeignKeyCascade(User, null=True, blank=True, related_name='creator_theme')
    isFolder = BooleanField(default=False)

    def __str__(self):
        return f'{self.code}: {self.name}'

    objects = Messages_theme_view_Manager()

    class Meta:
        verbose_name = 'Темы задач'
        managed = False
        db_table = 'tracker_messages_theme_view'



from django.urls import path

from clndr.views import day_types

urlpatterns = [

    path('Day_types/Fetch/', day_types.Day_types_Fetch),
    path('Day_types/Add', day_types.Day_types_Add),
    path('Day_types/Update', day_types.Day_types_Update),
    path('Day_types/Remove', day_types.Day_types_Remove),
    path('Day_types/Lookup/', day_types.Day_types_Lookup),
    path('Day_types/Info/', day_types.Day_types_Info),
    path('Day_types/Copy', day_types.Day_types_Copy),

]



import logging

from django.db.models import TextField, Model, UUIDField, BooleanField, FloatField, IntegerField, UniqueConstraint, Q

from isc_common.fields.related import ForeignKeyProtect
from one_c.models.param_type import Param_type

logger = logging.getLogger(__name__)


class Documents_param_1c(Model):
    type = ForeignKeyProtect(Param_type)
    value = TextField(db_index=True, null=True, blank=True)
    value_boolean = BooleanField(db_index=True, null=True, blank=True)
    value_float = FloatField(db_index=True, null=True, blank=True)
    value_int = IntegerField(db_index=True, null=True, blank=True)
    value_uuid = UUIDField(db_index=True, null=True, blank=True)

    def __str__(self):
        return f"(id: {self.id}, " \
               f"document: type: {self.type}, " \
               f"value: {self.value}, " \
               f"value_uuid: {self.value_uuid}, " \
               f"value_boolean: {self.value_boolean}, " \
               f"value_float: {self.value_float}, " \
               f"value_int: {self.value_int})"

    class Meta:
        verbose_name = 'Параметры документов 1С'
        constraints = [
            UniqueConstraint(fields=['type'], condition=Q(value=None) & Q(value_boolean=None) & Q(value_float=None) & Q(value_int=None) & Q(value_uuid=None), name='Documents_param_1c_unique_constraint_0'),
            UniqueConstraint(fields=['type', 'value'], condition=Q(value_boolean=None) & Q(value_float=None) & Q(value_int=None) & Q(value_uuid=None), name='Documents_param_1c_unique_constraint_1'),
            UniqueConstraint(fields=['type', 'value_boolean'], condition=Q(value=None) & Q(value_float=None) & Q(value_int=None) & Q(value_uuid=None), name='Documents_param_1c_unique_constraint_2'),
            UniqueConstraint(fields=['type', 'value', 'value_boolean'], condition=Q(value_float=None) & Q(value_int=None) & Q(value_uuid=None), name='Documents_param_1c_unique_constraint_3'),
            UniqueConstraint(fields=['type', 'value_float'], condition=Q(value=None) & Q(value_boolean=None) & Q(value_int=None) & Q(value_uuid=None), name='Documents_param_1c_unique_constraint_4'),
            UniqueConstraint(fields=['type', 'value', 'value_float'], condition=Q(value_boolean=None) & Q(value_int=None) & Q(value_uuid=None), name='Documents_param_1c_unique_constraint_5'),
            UniqueConstraint(fields=['type', 'value_boolean', 'value_float'], condition=Q(value=None) & Q(value_int=None) & Q(value_uuid=None), name='Documents_param_1c_unique_constraint_6'),
            UniqueConstraint(fields=['type', 'value', 'value_boolean', 'value_float'], condition=Q(value_int=None) & Q(value_uuid=None), name='Documents_param_1c_unique_constraint_7'),
            UniqueConstraint(fields=['type', 'value_int'], condition=Q(value=None) & Q(value_boolean=None) & Q(value_float=None) & Q(value_uuid=None), name='Documents_param_1c_unique_constraint_8'),
            UniqueConstraint(fields=['type', 'value', 'value_int'], condition=Q(value_boolean=None) & Q(value_float=None) & Q(value_uuid=None), name='Documents_param_1c_unique_constraint_9'),
            UniqueConstraint(fields=['type', 'value_boolean', 'value_int'], condition=Q(value=None) & Q(value_float=None) & Q(value_uuid=None), name='Documents_param_1c_unique_constraint_10'),
            UniqueConstraint(fields=['type', 'value', 'value_boolean', 'value_int'], condition=Q(value_float=None) & Q(value_uuid=None), name='Documents_param_1c_unique_constraint_11'),
            UniqueConstraint(fields=['type', 'value_float', 'value_int'], condition=Q(value=None) & Q(value_boolean=None) & Q(value_uuid=None), name='Documents_param_1c_unique_constraint_12'),
            UniqueConstraint(fields=['type', 'value', 'value_float', 'value_int'], condition=Q(value_boolean=None) & Q(value_uuid=None), name='Documents_param_1c_unique_constraint_13'),
            UniqueConstraint(fields=['type', 'value_boolean', 'value_float', 'value_int'], condition=Q(value=None) & Q(value_uuid=None), name='Documents_param_1c_unique_constraint_14'),
            UniqueConstraint(fields=['type', 'value', 'value_boolean', 'value_float', 'value_int'], condition=Q(value_uuid=None), name='Documents_param_1c_unique_constraint_15'),
            UniqueConstraint(fields=['type', 'value_uuid'], condition=Q(value=None) & Q(value_boolean=None) & Q(value_float=None) & Q(value_int=None), name='Documents_param_1c_unique_constraint_16'),
            UniqueConstraint(fields=['type', 'value', 'value_uuid'], condition=Q(value_boolean=None) & Q(value_float=None) & Q(value_int=None), name='Documents_param_1c_unique_constraint_17'),
            UniqueConstraint(fields=['type', 'value_boolean', 'value_uuid'], condition=Q(value=None) & Q(value_float=None) & Q(value_int=None), name='Documents_param_1c_unique_constraint_18'),
            UniqueConstraint(fields=['type', 'value', 'value_boolean', 'value_uuid'], condition=Q(value_float=None) & Q(value_int=None), name='Documents_param_1c_unique_constraint_19'),
            UniqueConstraint(fields=['type', 'value_float', 'value_uuid'], condition=Q(value=None) & Q(value_boolean=None) & Q(value_int=None), name='Documents_param_1c_unique_constraint_20'),
            UniqueConstraint(fields=['type', 'value', 'value_float', 'value_uuid'], condition=Q(value_boolean=None) & Q(value_int=None), name='Documents_param_1c_unique_constraint_21'),
            UniqueConstraint(fields=['type', 'value_boolean', 'value_float', 'value_uuid'], condition=Q(value=None) & Q(value_int=None), name='Documents_param_1c_unique_constraint_22'),
            UniqueConstraint(fields=['type', 'value', 'value_boolean', 'value_float', 'value_uuid'], condition=Q(value_int=None), name='Documents_param_1c_unique_constraint_23'),
            UniqueConstraint(fields=['type', 'value_int', 'value_uuid'], condition=Q(value=None) & Q(value_boolean=None) & Q(value_float=None), name='Documents_param_1c_unique_constraint_24'),
            UniqueConstraint(fields=['type', 'value', 'value_int', 'value_uuid'], condition=Q(value_boolean=None) & Q(value_float=None), name='Documents_param_1c_unique_constraint_25'),
            UniqueConstraint(fields=['type', 'value_boolean', 'value_int', 'value_uuid'], condition=Q(value=None) & Q(value_float=None), name='Documents_param_1c_unique_constraint_26'),
            UniqueConstraint(fields=['type', 'value', 'value_boolean', 'value_int', 'value_uuid'], condition=Q(value_float=None), name='Documents_param_1c_unique_constraint_27'),
            UniqueConstraint(fields=['type', 'value_float', 'value_int', 'value_uuid'], condition=Q(value=None) & Q(value_boolean=None), name='Documents_param_1c_unique_constraint_28'),
            UniqueConstraint(fields=['type', 'value', 'value_float', 'value_int', 'value_uuid'], condition=Q(value_boolean=None), name='Documents_param_1c_unique_constraint_29'),
            UniqueConstraint(fields=['type', 'value_boolean', 'value_float', 'value_int', 'value_uuid'], condition=Q(value=None), name='Documents_param_1c_unique_constraint_30'),
            UniqueConstraint(fields=['type', 'value', 'value_boolean', 'value_float', 'value_int', 'value_uuid'], name='Documents_param_1c_unique_constraint_31'),
        ]



# Generated by Django 2.2.5 on 2019-09-09 10:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twits', '0013_chat_usersview'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='chat_usersview',
            options={'managed': False, 'verbose_name': 'Участники чатов'},
        ),
    ]



from django.urls import path
from isc_common.auth.views.user_group import UserGroup_Fetch, UserGroup_Add, UserGroup_Update, UserGroup_Remove, UserGroup_Info, UserGroup_Lookup

urlpatterns = [

    path('UserGroups/Fetch/', UserGroup_Fetch),
    path('UserGroups/Add', UserGroup_Add),
    path('UserGroups/Update', UserGroup_Update),
    path('UserGroups/Remove', UserGroup_Remove),
    path('UserGroups/Lookup/', UserGroup_Lookup),
    path('UserGroups/Info/', UserGroup_Info),

]



import logging

from isc_common import setAttr, delAttr
from isc_common.http.RPCRequest import RPCRequest
from isc_common.json import BytesToJson

logger = logging.getLogger(__name__)


class RequestData:
    def __init__(self, _dict=None, request=None):
        if request:
            self.__data_dict__ = request.get_data()
        elif _dict:
            self.__data_dict__ = _dict
        else:
            self.__data_dict__ = dict()
        super()

    def getDataOfKey(self, key, default=None):
        return self.__data_dict__.get(key, default)

    def update(self, key, value):
        self.__data_dict__.update((key, value))
        return RequestData(self.__data_dict__)

    def getDataWithOutField(self, list_fields):
        if isinstance(list_fields, list):
            return RequestData({key: self.__data_dict__[key] for key in self.__data_dict__ if not key in list_fields})
        else:
            return RequestData(self.__data_dict__)

    def getDataNotContain(self, *sub_strs):
        res = RequestData()
        for sub in sub_strs:
            self.__data_dict__ = {key: self.__data_dict__[key] for key in self.__data_dict__ if not str(key).__contains__(sub)}
        return RequestData(self.__data_dict__)

    def dict(self):
        return self.__data_dict__


class DSRequest(RPCRequest):
    alive_only = None
    tag = None
    begdate = None
    dataPageSize = 75
    drawAheadRatio = 1.2
    enabledAll = None
    enddate = None
    endRow = None
    startRow = None
    visibleMode = 'none'

    def __init__(self, request):
        from isc_common.auth.models.user import User

        self.json = None

        if request != None and request.body != b'':
            self.json = BytesToJson(request.body)

            data = self.json.get('data')
            self.user = None
            if isinstance(data, dict):
                httpHeaders = data.get('httpHeaders')
                if isinstance(httpHeaders, dict):
                    if httpHeaders.get('WS_CHANNEL') != None:
                        self.ws_channel = httpHeaders.get('WS_CHANNEL')

                    if httpHeaders.get('WS_PORT') != None:
                        self.ws_port = httpHeaders.get('WS_PORT')

                    if httpHeaders.get('HOST') != None:
                        self.host = httpHeaders.get('HOST')

                    if httpHeaders.get('USERNAME') != None:
                        self.username = httpHeaders.get('USERNAME')

                    if httpHeaders.get('USER_ID') != None:
                        # print(f'''USER_ID: {httpHeaders.get('USER_ID')}''')
                        self.user_id = httpHeaders.get('USER_ID')
                        self.user = User.objects.get(id=self.user_id)

                    if httpHeaders.get('IS_ADMIN') != None:
                        self.is_admin = httpHeaders.get('IS_ADMIN')

                    if httpHeaders.get('IS_DEVELOP') != None:
                        self.is_develop = httpHeaders.get('IS_DEVELOP')

                    delAttr(self.json.get('data'), 'httpHeaders')

            if self.user == None:
                self.username = request.session._session.get('username')
                self.user_id = request.session._session.get('user_id')

                try:
                    if self.user_id == None and request.GET.get('ws_channel') != None:
                        self.user_id = User.objects.get(username=request.GET.get('ws_channel').split('_')[1]).id
                    if self.user_id != None:
                        self.user = User.objects.get(id=self.user_id)
                    else:
                        self.user = None
                except User.DoesNotExist:
                    self.user = None
                self.is_admin = request.session._session.get('is_admin')
                self.is_develop = request.session._session.get('is_develop')
                self.ws_channel = request.session._session.get('ws_channel')
                self.ws_port = request.session._session.get('ws_port')
                self.host = request.session._session.get('host')

        RPCRequest.__init__(self, self.json)

    def get_data(self, excluded_keys=['grid']):
        res = None
        if isinstance(self.json, dict):
            data = self.json.get('data')
            if isinstance(data, dict):
                res = data
            elif isinstance(data, str):
                return data
            else:
                res = self.json

            res = dict((key, value) for (key, value) in res.items() if not key.startswith('_') and key not in excluded_keys)
        return res

    def set_data(self, data):
        if isinstance(data, dict):
            setAttr(self.json, 'data', data)

    def get_criteria(self, excluded_keys=['grid']):
        data = self.get_data(excluded_keys=excluded_keys)
        criteria = data.get('criteria')
        if isinstance(criteria, list):
            return criteria
        else:
            return []

    def get_data_array(self, excluded_keys=['grid']):
        res = None
        if isinstance(self.json, dict):
            transaction = self.json.get('transaction', None)
            if transaction:
                operations = transaction.get('operations')
            else:
                return (False, [self.get_data()])

            res = (True, [dict((key, value) for (key, value) in operation.items() if not str(key).startswith('_') and key not in excluded_keys) for operation in operations])
            # res = (True, [dict((key, value) for (key, value) in operation.items() if value is not None and not str(key).startswith('_') and key not in excluded_keys) for operation in operations])
        return res

    def get_oldValues(self, excluded_keys=['grid']):
        res = None
        if isinstance(self.json, dict):
            if self.json.get('oldValues', None) and isinstance(self.json.get('oldValues', None), dict):
                res = self.json.get('oldValues')
            else:
                res = self.json

            res = dict((key, value) for (key, value) in res.items() if not str(key).startswith('_') and key not in excluded_keys)
            # res = dict((key, value) for (key, value) in res.items() if value is not None and not str(key).startswith('_') and key not in excluded_keys)
        return res

    def get_id(self):
        return self.get_data().get('id', self.get_oldValues().get('id'))

    def get_operationtype(self):
        return self.json.get('operationType')

    def get_username(self):
        return self.get_data().get('username')

    def get_ids(self):
        multi, data_array = self.get_data_array()
        if data_array:
            if multi:
                ids = [record.get('data').get('id') for record in data_array]
            else:
                ids = [record.get('data').get('id') if record.get('data') else record.get('id') for record in data_array]
        else:
            ids = [0]
        return [id for id in ids if id is not None]

    def get_tuple_ids(self):
        multi, data_array = self.get_data_array()
        if data_array:
            if multi:
                ids = [(record.get('data').get('id'), record.get('visibleMode', 'none')) for record in data_array]
            else:
                ids = [(record.get('data').get('id'), self.visibleMode) if record.get('data') else (record.get('id'), self.visibleMode) for record in data_array]
        else:
            ids = [(0, "none")]
        return ids

    def get_old_ids(self):
        multi, data_array = self.get_data_array()
        if data_array:
            if multi:
                ids = [record.get('oldValues').get('id') for record in data_array]
            else:
                ids = [record.get('oldValues').get('id') if record.get('oldValues') else record.get('id') for record in data_array]
        else:
            ids = [0]
        return [id for id in ids if id is not None]

    def __str__(self):
        return str(self.__dict__)



# Generated by Django 2.2.4 on 2019-08-25 10:06

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0054_shift_day_relax_color'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='calendar_shifts_days',
            name='saturday',
        ),
        migrations.AddField(
            model_name='shifts',
            name='order',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='calendar_shifts_days',
            name='friday',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, related_name='saturday_rel', to='clndr.Shift_day'),
        ),
    ]



import logging
import time
import uuid
from datetime import datetime

from django.conf import settings
from django.db import transaction
from django.utils import timezone

from history.models.visitor import Visitor
from history.utils import get_ip
from isc_common import setAttr
from isc_common.auth.models.user import User
from isc_common.bit import IsBitOn, TurnBitOn, TurnBitOff
from isc_common.datetime import DateToStr
from isc_common.http.DSRequest import DSRequest
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.ws.webSocket import WebSocket
from kaf_pas.planing.models.status_operation_types import Status_operation_types
from tracker.models.messages_state import Messages_state
from tracker.models.messages_theme import Messages_theme
from twits.models.chat_user_user import Chat_user_user
from twits.models.chat_user_user_theme import Chat_user_user_theme
from twits.models.chat_users_view import Chat_usersView

logger = logging.getLogger(__name__)


class LoginRequest(DSRequest):
    def __init__(self, request):
        from reports.models.jasper_reports_users import Jasper_reports_users
        from django.forms import model_to_dict

        DSRequest.__init__(self, request)
        data = self.get_data()
        login = data.get('login', None)
        errorMessage = "Аутентификация не прошла :-("

        try:
            user = User.objects.get(username=login)

            if user.check_password(data.get('password', None)):
                ws_channel = f'{settings.WS_CHANNEL}_{login}'

                ip = get_ip(request)

                with transaction.atomic():
                    Visitor.objects.using('history').select_for_update()
                    # visitors_all = [visitor for visitor in Visitor.objects.using('history').filter(username=login)]
                    # if len(visitors_all) > 1:
                    #     Visitor.objects.using('history').filter(username=login).delete()

                    if settings.FREQUENCY_OF_POLLING_UNREAD_MESSAGES == None:
                        settings.FREQUENCY_OF_POLLING_UNREAD_MESSAGES = 1000 * 60 * 3

                    visitors = [visitor.ip_address for visitor in Visitor.objects.using('history').filter(username=login).exclude(ip_address=ip)]
                    if len(visitors) > 0:
                        ip_address = visitors[0]
                        if ip_address != ip:
                            self.response = dict(status=RPCResponseConstant.statusLoginIncorrect, errorMessage=f'Вход с логином "{login}" уже выполнен на {ip_address}')
                    else:

                        operations_statuses = []
                        for status_operation_types in Status_operation_types.objects.all():
                            operations_statuses.append(
                                dict(
                                    opertype_id=status_operation_types.opertype.id,
                                    opertype__code=status_operation_types.opertype.code,
                                    status_id=status_operation_types.id,
                                    status__code=status_operation_types.code,
                                    status__name=status_operation_types.name,
                                ))

                        from kaf_pas.ckk.models.locations_users import Locations_users
                        from kaf_pas.ckk.models.locations_view import Locations_view
                        if user.is_admin or user.is_develop:
                            workshops=[]
                        else:
                            workshops = [Locations_view.objects.get(id=Locations_users.location.id).workshop for Locations_users in Locations_users.objects.filter(user=user)]
                            workshops = [workshop.id if workshop else 0 for workshop in workshops]
                            if len(workshops) == None:
                                workshops = [-1]

                        self.response = dict(
                            captionUser=user.get_short_name,
                            chatInfo=LoginRequest.get_chats(user),
                            codeGroup="",
                            dynDataSources=settings.DYNAMIC_CLASS.get_datasources(),
                            fio=user.get_short_name,
                            frequencyOfPollingUnreadMessages=settings.FREQUENCY_OF_POLLING_UNREAD_MESSAGES,
                            isAdmin=user.is_admin,
                            isDevelop=user.is_develop,
                            jsLogDebug=settings.JS_LOG_DEBUG if settings.JS_LOG_DEBUG == 1 else 0,
                            KOMPAS_INFORMICA_WS=settings.KOMPAS_INFORMICA_WS,
                            login=login,
                            message_state_delivered_id=Messages_state.message_state_delivered().id,
                            message_state_delivered_name=Messages_state.message_state_delivered().name,
                            message_state_new_id=Messages_state.message_state_new().id,
                            message_state_new_name=Messages_state.message_state_new().name,
                            message_state_not_readed_id=Messages_state.message_state_not_readed().id,
                            message_state_not_readed_name=Messages_state.message_state_not_readed().name,
                            message_state_readed_id=Messages_state.message_state_readed().id,
                            message_state_readed_name=Messages_state.message_state_readed().name,
                            Operations_Plan=settings.OPERATIONS_PLAN.OPERSTYPES,
                            # progresses=[ProgressesManager.getRecord(item) for item in Progresses.objects.filter(user=user)],
                            status=RPCResponseConstant.statusSuccess,
                            user__color=user.color if user.color != None and user.color != 'undefined' else 'black',
                            user_full_name=user.get_full_name,
                            user_short_name=user.get_short_name,
                            workshops=workshops,
                            userId=user.id,
                            ws_channel=ws_channel,
                            ws_host=settings.WS_HOST,
                            ws_port=settings.WS_PORT,
                            operations_statuses=operations_statuses,
                            jasper_reports=[dict(
                                report=model_to_dict(item.report),
                                editor_identifier=item.editor_identifier,
                            ) for item in Jasper_reports_users.objects.filter(user=user)]
                        )
                        request.session['ws_channel'] = ws_channel
                        request.session['ws_port'] = settings.WS_PORT
                        request.session['host'] = settings.WS_HOST
                        request.session['username'] = user.username
                        request.session['fio'] = user.get_short_name
                        request.session['user_id'] = user.id
                        request.session['is_admin'] = user.is_admin
                        request.session['is_develop'] = user.is_develop
                        user.last_login = datetime.now()
                        user.save()

            else:
                self.response = dict(status=RPCResponseConstant.statusLoginIncorrect, errorMessage=errorMessage)

        except User.DoesNotExist:
            self.response = dict(status=RPCResponseConstant.statusLoginIncorrect, errorMessage=errorMessage)

    @staticmethod
    def get_chats(user):
        tetatet_chats_reciver = []
        tetatet_chats_sender = []
        tetatet_chats = []

        key = 'LoginRequest.get_chats'
        settings.LOCKS.acquire(key)
        for chat in Chat_user_user.objects.filter(user_reciver=user):
            tetatet_chats_reciver.append(dict(
                id=chat.common_id,
                props=chat.props._value,
                sender_id=chat.user_sender.id,
                fullname=chat.user_sender.get_full_name,
                shortname=chat.user_sender.get_short_name,
                theme_ids=[item.theme.id for item in Chat_user_user_theme.objects.filter(chat_user_user__common_id=chat.common_id)],
                view_theme_ids=[item.theme.id for item in Chat_user_user_theme.objects.filter(chat_user_user__common_id=chat.common_id) if item.chat_user_user.user_sender == user],
                readonly=True
            ))

        for chat in Chat_user_user.objects.filter(user_sender=user):
            tetatet_chats_sender.append(dict(
                id=chat.common_id,
                props=chat.props._value,
                reciver_id=chat.user_reciver.id,
                fullname=chat.user_reciver.get_full_name,
                shortname=chat.user_reciver.get_short_name,
                theme_ids=[item.theme.id for item in Chat_user_user_theme.objects.filter(chat_user_user__common_id=chat.common_id)],
                view_theme_ids=[item.theme.id for item in Chat_user_user_theme.objects.filter(chat_user_user__common_id=chat.common_id) if item.chat_user_user.user_sender == user],
                readonly=False
            ))

        for chat in tetatet_chats_reciver:
            chat2 = [item for item in tetatet_chats_sender if item.get('id') == chat.get('id')]
            read_only = len(chat2) == 0
            if read_only:
                tetatet_chats.append(chat)
            else:
                compulsory_reading = IsBitOn(chat.get('props'), 0)
                if compulsory_reading:
                    props = TurnBitOn(chat2[0].get('props'), 0)
                else:
                    props = TurnBitOff(chat2[0].get('props'), 0)
                setAttr(chat2[0], 'props', props)
                tetatet_chats.append(chat2[0])

        for chat in tetatet_chats_sender:
            chat2 = [item for item in tetatet_chats if item.get('id') == chat.get('id')]
            if len(chat2) == 0:
                tetatet_chats.append(chat)

        group_chats = [
            dict(
                id=chat_users.chat.id,
                props=chat_users.props._value,
                code=chat_users.chat.code,
                name=chat_users.chat.name,
                readonly=chat_users.readonly,
                theme_ids=[
                    Messages_theme.objects.get_or_create(
                        code=f'group_{chat_users.chat.code}',
                        defaults=dict(
                            name=f'Чат: {chat_users.chat.name}',
                            creator=User.admin_user(),
                            editing=False,
                            deliting=False,
                            parent=Messages_theme.group_chats_theme(),
                            description='Группа создана автоматически'))[0].id
                ],
            ) for chat_users in Chat_usersView.objects.filter(user=user)]
        settings.LOCKS.release(key)

        return dict(
            group_chats=group_chats,
            tetatet_chats=tetatet_chats,
        )

    @staticmethod
    def send_bot_message(user, bot, message, compulsory_reading=False):
        _user = user
        _bot = bot

        key = 'LoginRequest.send_bot_message'
        settings.LOCKS.acquire(key)
        if not isinstance(_user, User) and isinstance(_user, int):
            _user = User.objects.get(id=_user)

        if not isinstance(_bot, User) and isinstance(_bot, int):
            _bot = User.objects.get(id=_bot)

        props = TurnBitOn(0, 0) if compulsory_reading else TurnBitOff(0, 0)
        chat_user_user, created = Chat_user_user.objects.get_or_create(
            user_sender=_bot,
            user_reciver=_user,
            defaults=dict(props=props)
        )

        def send():
            channel = f'tetatet_{chat_user_user.common_id}'

            _message = dict(
                channel=channel,
                date=DateToStr(timezone.now(), '%d.%m.%Y, %H:%M:%S', 3),
                guid=str(uuid.uuid4()).upper(),
                host=settings.WS_HOST,
                message=f'<pre>{message}</pre>',
                message_state_delivered_id=Messages_state.message_state_delivered().id,
                message_state_delivered_name=Messages_state.message_state_delivered().name,
                message_state_new_id=Messages_state.message_state_new().id,
                message_state_new_name=Messages_state.message_state_new().name,
                message_state_readed_id=Messages_state.message_state_readed().id,
                message_state_readed_name=Messages_state.message_state_readed().name,
                message_state_not_readed_id=Messages_state.message_state_not_readed().id,
                message_state_not_readed_name=Messages_state.message_state_not_readed().name,
                path='twit',
                port=settings.WS_PORT,
                props=0,
                theme_ids=[item.theme.id for item in Chat_user_user_theme.objects.filter(chat_user_user=chat_user_user)],
                type='message',
                user__color=_bot.color if _bot.color else 'black',
                user__full_name=_bot.get_full_name,
                user__short_name=_bot.get_short_name,
                user_id=_bot.id
            )

            WebSocket.send_message(
                host=settings.WS_HOST,
                port=settings.WS_PORT,
                channel=channel,
                message=_message,
                logger=logger
            )

        if created:
            from twits.models.chats import ChatsManager
            ChatsManager.refresh_chat_menu(user=_user, logger=logger)
            time.sleep(3)
            send()
        else:
            if chat_user_user.props._value != props:
                chat_user_user.props = props
                chat_user_user.save()

                from twits.models.chats import ChatsManager
                ChatsManager.refresh_chat_menu(user=_user, logger=logger)
                time.sleep(3)
            send()
        settings.LOCKS.release(key)



# Generated by Django 2.2.8 on 2019-12-04 11:33

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0070_auto_20190829_1017'),
    ]

    operations = [
        migrations.AddField(
            model_name='calendars',
            name='props',
            field=bitfield.models.BitField((('default', 'Текущий календарь (по-умолчанию)'),), db_index=True, default=0),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-13 20:12

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0017_auto_20190421_1501'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Messages_view',
        ),
    ]



import logging

from django.db.models import Model

from isc_common.fields.related import ForeignKeyCascade
from one_c.models.document_1c import Document_1c
from one_c.models.documents_param_1c import Documents_param_1c

logger = logging.getLogger(__name__)


class Documents_param_cross_1c(Model):
    document = ForeignKeyCascade(Document_1c)
    param = ForeignKeyCascade(Documents_param_1c)

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Кросс таблица'
        unique_together = (('document', 'param'),)



import datetime
import logging

from django.contrib.postgres.fields import JSONField
from django.db import connections
from django.db.models import Model, DateTimeField, Manager, CharField, TextField
from django.utils import timezone

from isc_common.auth.models.user import User
from isc_common.datetime import StrToDate
from isc_common.fields.code_field import CodeField, CodeStrictField
from isc_common.fields.name_field import NameField
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_manager import CommonQuerySet

logger = logging.getLogger(__name__)


class HistoryQuerySet(CommonQuerySet):
    def get_range_rows1(self, request, function=None, distinct_field_names=None):
        request = DSRequest(request=request)

        data = request.get_data()
        begdate = None
        enddate = None
        if data.get('begdate'):
            begdate = StrToDate(data.get('begdate'), '%Y-%m-%d')
        if data.get('enddate'):
            enddate = StrToDate(data.get('enddate'), '%Y-%m-%d')

        with connections['history'].cursor() as cursor:
            if not begdate and not enddate:
                cursor.execute('''select
                                    username,
                                    fio,
                                    count(*)
                                from history_history
                                group by username, fio 
                                order by fio''')
            elif begdate and not enddate:
                cursor.execute('''select
                                        username,
                                        fio,
                                        count(*)
                                    from history_history
                                    where date >= %s
                                    group by username, fio
                                    order by fio''', [begdate])
            elif not begdate and enddate:
                cursor.execute('''select
                                        username,
                                        fio,
                                        count(*)
                                    from history_history
                                    where date <= %s
                                    group by username, fio
                                    order by fio''', [enddate])
            else:
                cursor.execute('''select
                                    username,
                                    fio,
                                    count(*)
                                from history_history
                                where date >= %s and date <= %s
                                group by username, fio
                                order by fio''', [begdate, enddate])
            count_blocks = cursor.fetchall()
            res = dict(
                data=[dict(username=count_block[0], fio=count_block[1], count=count_block[2]) for count_block in count_blocks],
                colors=[dict(fio=user.get_short_name, color=user.color) for user in User.objects.all() if user.color]
            )
            return res


class HistoryManager(Manager):

    def get_queryset(self):
        return HistoryQuerySet(self.model, using=self._db)


class History(Model):
    date = DateTimeField(db_index=True, default=timezone.now)
    method = CodeField()
    path = CodeStrictField()
    username = NameField()
    fio = NameField()
    data = JSONField(default=dict)
    ip_address = CharField(max_length=20)
    user_agent = TextField()

    objects = HistoryManager()

    def __str__(self):
        return f"method: {self.method} path: {self.path} data{self.data}"

    class Meta:
        verbose_name = 'History'



import logging

from django.contrib.sessions.models import Session
from django.db.models import CharField

from isc_common.auth.models.user import User
from isc_common.fields.related import ForeignKeyCascade
from isc_common.models.audit import AuditManager, AuditModel, AuditQuerySet

logger = logging.getLogger(__name__)


class SessionIPSQuerySet(AuditQuerySet):
    pass


class SessionIPSManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
        }
        return res

    def get_queryset(self):
        return SessionIPSQuerySet(self.model, using=self._db)


class SessionIPS(AuditModel):
    ip = CharField(max_length=20)
    session = ForeignKeyCascade(Session)
    user = ForeignKeyCascade(User)

    objects = SessionIPSManager()

    def __str__(self):
        return f"ID:{self.id}, code: {self.code}, name: {self.name}, description: {self.description}"

    class Meta:
        verbose_name = 'Ссессти и пользователи'



# Generated by Django 3.0.3 on 2020-02-05 12:37

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0077_messages_theme_with_out_user_view'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Messages_theme_with_out_user_view',
        ),
    ]



import json

from isc_common.string import StrToBytes, BytesToStr


def BytesToJson(mybytes):
    mystring = BytesToStr(mybytes)
    _json = json.loads(mystring)
    return _json


def JsonToBytes(meyjson):
    mystring = json.dumps(meyjson)
    bytes = StrToBytes(mystring)
    return bytes



import datetime
import logging

from django.db.models import TextField
from django.forms import model_to_dict
from django.utils import timezone

from isc_common import setAttr
from isc_common.auth.models.user import User
from isc_common.datetime import StrToDate
from isc_common.fields.related import ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditModel, AuditQuerySet, AuditManager

logger = logging.getLogger(__name__)


class ParamsQuerySet(AuditQuerySet):
    def get_params(self, request, *args):
        request = DSRequest(request=request)
        data = request.get_data()
        app_id = data.get('app_id')

        periodEraseHistory = int(super().get_or_create(user=request.user, key=f'periodEraseHistory{app_id}', defaults=dict(value=1))[0].value)

        lastEraseHistory = super().get_or_create(user=request.user, key=f'lastEraseHistory{app_id}', defaults=dict(value=timezone.now()))[0].value
        lastEraseHistory = StrToDate(lastEraseHistory, '%Y-%m-%d %H:%M:%S')
        delta = timezone.now() - lastEraseHistory

        if not delta or delta.days > periodEraseHistory:
            super().filter(user=request.user, key__icontains='_ComboBoxItemSS').delete()
            if not lastEraseHistory:
                super().create(user=request.user, key=f'lastEraseHistory{app_id}', value=datetime.datetime.now())
            else:
                super().filter(user=request.user, key=f'lastEraseHistory{app_id}', value=lastEraseHistory).update(value=timezone.now())

        queryResult = super().filter(user_id=data.get('user_id'))
        res = [model_to_dict(record) for record in queryResult]
        return res

    def get_params_key(self, request, *args):
        request = DSRequest(request=request)
        data = request.get_data()
        key = data.get('key')

        try:
            return Params.objects.get(user=request.user, key=key).value
        except Params.DoesNotExist:
            return dict()


class ParamsManager(AuditManager):
    def get_queryset(self):
        return ParamsQuerySet(self.model, using=self._db)

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'key': record.key[1:len(record.key) - 1] if record.key[0:1] == "'" else record.key,
            'value': record.value,
            'lastmodified': record.lastmodified,
        }
        return res

    def deleteFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        ids = []
        if data:
            ids = [param.id for param in Params.objects.filter(user=data.get('user_id'), key=data.get('key'))]

        if len(ids) == 0:
            ids = request.get_ids()
        res = super().filter(id__in=ids).delete()
        return res[0]

    def update1FromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        key = data.get('key')
        value = data.get('value')
        user_id = data.get('user_id')

        res = data
        if key != None and user_id != None:
            if key[0:1] == "'":
                key = key[1:len(key) - 1]
            setAttr(data, 'key', key)

            if value == None or value == '':
                res = super().filter(
                    user_id=user_id,
                    key=key
                ).delete()
            else:
                res, _ = super().update_or_create(
                    user_id=user_id,
                    key=key,
                    defaults=dict(value=value)
                )
                res = model_to_dict(res)
        return res


class Params(AuditModel):
    user = ForeignKeyCascade(User)
    key = TextField()
    value = TextField(null=True, blank=True)

    objects = ParamsManager()

    def __str__(self):
        return f"{self.key}: {self.value}"

    class Meta:
        verbose_name = 'Сохраненные параметры'
        unique_together = (('user', 'key'),)



import json
import logging

from django.conf import settings
from django.utils.deprecation import MiddlewareMixin

from history import utils
from history.models.history import History

logger = logging.getLogger(__name__)


class LoggingRequestMiddleware(MiddlewareMixin):
    def process_request(self, request):
        try:
            bodyStr = request.body.decode("utf-8")
            self.json = json.loads(bodyStr)

        except:
            self.json = None

        if settings.INCLUDE_REQUEST_PATHES == None:
            settings.INCLUDE_REQUEST_PATHES = ['Add', 'Update', 'Remove']

        username = request.session._session.get('username')
        fio = request.session._session.get('fio', 'unknown')
        if username != None:
            path = request.path.split('/')
            path = path[len(path) - 1]
            enable_logging = path in settings.INCLUDE_REQUEST_PATHES
            if enable_logging and self.json != None:
                ip_address = utils.get_ip(request)
                user_agent = request.META.get('HTTP_USER_AGENT')

                logger.debug(f'request.path : {request.path}')
                history_element = History.objects.using('history').create(
                    username=username,
                    fio=fio,
                    method=request.method,
                    ip_address=ip_address,
                    user_agent=user_agent,
                    path=request.path,
                    data=self.json
                )

                logger.debug('=========================================================================================================')
                logger.debug(f'date: {history_element.date}')
                logger.debug(f'username: {username}')
                logger.debug(f'method: {request.method}')
                logger.debug(f'path: {request.path}')
                logger.debug(f'ip_address: {ip_address}')
                logger.debug(f'user_agent: {user_agent}')
                logger.debug(f'json: {json.dumps(self.json, indent=4, sort_keys=True)}')
                logger.debug('=========================================================================================================\n')

    def process_response(self, request, response):
        return response



from decimal import Decimal

from bitfield import BitHandler, Bit

from isc_common import setAttr


def DecimalToStr(value):
    if value == None:
        return ''

    try:
        res = str(value)
        res.split('.')
        cel = res.split('.')[0]
        float = StrToNumber(res.split('.')[1:][0])
        if float > 0:
            return f'{cel}.{float}'
        return cel
    except IndexError:
        return value


def StrToNumber(s1):
    if s1 == None:
        return None

    if not isinstance(s1, str):
        raise Exception(f'{s1} is not str type.')

    s = s1.replace(',', '.')

    if not isinstance(s, str):
        raise Exception(f'{s} is not str type.')
    try:
        return int(s)
    except ValueError:
        try:
            return float(s)
        except ValueError as ex:
            return None


def IntToNumber(s):
    if s == None:
        return None

    if not isinstance(s, int):
        raise Exception(f'{s} is not int type.')

    return float(s)

def ToNumber(s):
    if s == None:
        return 0

    if isinstance(s, int):
        return IntToNumber(s)
    elif isinstance(s, str):
        return StrToNumber(s)
    elif isinstance(s, float):
        return s
    elif isinstance(s, Decimal):
        return float(s)
    else:
        raise Exception(f'{s} unknown type.')

def ToDecimal(s):
    if s == None:
        return 0

    if isinstance(s, int):
        return Decimal(s)
    elif isinstance(s, str):
        return Decimal(s)
    elif isinstance(s, float):
        return Decimal(s)
    elif isinstance(s, Decimal):
        return s
    else:
        raise Exception(f'{s} unknown type.')


def IntToDecimal(s):
    if s == None:
        return None

    if not isinstance(s, int):
        raise Exception(f'{s} is not int type.')

    return Decimal(s)


def StrToInt(s):
    if s == None:
        return None
    try:
        return int(s)
    except ValueError as ex:
        return None


def DelProps(value):
    if isinstance(value, dict):
        for key, _value in value.items():
            if isinstance(_value, BitHandler):
                setAttr(value, key, _value._value)
            elif isinstance(_value, Bit):
                setAttr(value, key, _value.is_set)
        return value
    else:
        value


def GetPropsInt(value):
    if isinstance(value, BitHandler):
        return value._value
    else:
        value



# Generated by Django 2.2.4 on 2019-08-24 19:44

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0052_auto_20190824_1940'),
    ]

    operations = [
        migrations.AlterField(
            model_name='holidays',
            name='calendar',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='clndr.Calendars'),
        ),
    ]



# Generated by Django 2.1.7 on 2019-02-16 23:23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0006_auto_20190216_2317'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='user_permission',
            unique_together={('user', 'widget')},
        ),
        migrations.AlterUniqueTogether(
            name='usergroup_permission',
            unique_together={('usergroup', 'widget')},
        ),
    ]



import hashlib
import logging
import uuid
from tempfile import TemporaryFile

from django.core.files import File
from django.db import transaction

from crypto.models.upload_file import DSResponse_UploadFile
from isc_common.auth.models.user import User
from isc_common.common.functions import Common
from isc_common.datetime import StrToDate
from isc_common.dropzone import Dz
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.ws.webSocket import WebSocket
from kaf_pas.common.UploadItem import UploadItem
from tracker.models.messages import Messages
from twits import ChatResponseData
from twits.models.chat_user_user import Chat_user_user
from twits.models.chat_user_user_theme import Chat_user_user_theme
from twits.models.chat_users import Chat_users

logger = logging.getLogger(__name__)


class DSResponse__Messages_UploadFile(DSResponse_UploadFile):

    def __init__(self, request):

        self.dz_dictionary = Dz(request.POST)

        self.host = self.dz_dictionary.host
        self.port = self.dz_dictionary.port
        self.channel = self.dz_dictionary.channel

        self.file = request.FILES.get('file')
        self.user = User.objects.get(id=self.dz_dictionary.user_id)

        setattr(self.dz_dictionary, 'real_file_name', self.file.name)
        setattr(self.dz_dictionary, 'stored_file_name', self.dz_dictionary.dzuuid)
        setattr(self.dz_dictionary, 'file_size', self.dz_dictionary.dztotalfilesize)
        setattr(self.dz_dictionary, 'file_mime_type', self.file.content_type)
        setattr(self.dz_dictionary, 'guid', str(uuid.uuid4()).upper())

        self.handle_uploaded_file()

    @property
    def first_chunk(self):
        res = int(self.dz_dictionary.dzchunkindex) == 0
        return res

    @property
    def last_chunk(self):
        res = int(self.dz_dictionary.dztotalchunkcount) == int(self.dz_dictionary.dzchunkindex) + 1
        return res

    def handle_uploaded_file(self):
        try:

            item = UploadItem(dictionary=self.dz_dictionary.__dict__)

            def load_str(pers):
                return f'Загружено: {self.file.name}: {pers} %'

            with TemporaryFile() as src:
                src.seek(int(self.dz_dictionary.dzchunkbyteoffset))
                src.write(self.file.read())

                pers = round((int(self.dz_dictionary.dzchunkindex) * 100) / int(self.dz_dictionary.dztotalchunkcount), 2)
                if self.last_chunk:
                    pers = 100

                logger.debug(f'{load_str(pers)}, шаг : {int(self.dz_dictionary.dzchunkindex) + 1} из {self.dz_dictionary.dztotalchunkcount}')

                if self.last_chunk:
                    logger.debug(f'Загружен файл: {item.real_file_name}')

                    logger.debug(f'Запись временного файла.')
                    with open(item.full_path, 'w+b') as destination:
                        src.seek(0)
                        destination.write(src.read())
                    logger.debug(f'Запись временного файла выполнена.')

                    logger.debug(f'Запись: {item.full_path}.')
                    with open(item.full_path, 'rb') as src:
                        fileObj = File(src)

                        def rec_message(item):
                            # host, port, ws_channel данные для передачи ошибки в случае ее возникновения
                            url = f'logic/Messages_files/Download/{item.messages_file.id}/?host={self.host}&port={self.port}&ws_channel=common_{self.user.username}'
                            message = f'<div>Выложен файл: <a href="{url}" target="hiddenframe"><strong>{item.real_file_name}</strong></a>   {Common.get_size_file_str(item.file_size)}</div><p/>'

                            face = "times new roman,times,serif"
                            size = 4

                            message = f'<font face ="{face}" size="{size}">{message}</font>'

                            def _rec_message(id, item):
                                # if item.user_id != id:
                                if not isinstance(item.theme_ids, list):
                                    if isinstance(item.theme_ids, str):
                                        item.theme_ids = [int(item) for item in item.theme_ids.split(',')]
                                    else:
                                        item.theme_ids = [item.theme_ids]

                                for theme_id in item.theme_ids:
                                    message_obj = None
                                    try:
                                        chat_user_user_theme = Chat_user_user_theme.objects.get(theme_id=theme_id)
                                        if chat_user_user_theme.chat_user_user.user_reciver.id == id:
                                            message_obj, _ = Messages.objects.get_or_create(
                                                defaults=dict(
                                                    message=message,
                                                ),
                                                checksum=hashlib.md5(message.encode()).hexdigest(),
                                                guid=item.guid,
                                                state_id=item.message_state_new_id,
                                                theme_id=theme_id,
                                                to_whom_id=id,
                                                user_id=item.user_id,
                                            )
                                    except Chat_user_user_theme.DoesNotExist:
                                        message_obj, _ = Messages.objects.get_or_create(
                                            defaults=dict(
                                                message=message,
                                            ),
                                            checksum=hashlib.md5(message.encode()).hexdigest(),
                                            guid=item.guid,
                                            state_id=item.message_state_new_id,
                                            theme_id=theme_id,
                                            to_whom_id=id,
                                            user_id=item.user_id,
                                        )

                                    from tracker.models.messages_files_refs import Messages_files_refs
                                    if message_obj:
                                        Messages_files_refs.objects.get_or_create(messages_file=item.messages_file, message=message_obj)

                            type_chat, id = self.channel.split('_')

                            if type_chat == 'group':
                                for chat_users in Chat_users.objects.filter(chat_id=id):
                                    _rec_message(chat_users.user.id, item)

                            elif type_chat == 'tetatet':
                                for chat_user_user in Chat_user_user.objects.filter(common_id=id):
                                    _rec_message(chat_user_user.user_reciver.id, item)

                            message = ChatResponseData(
                                channel=self.channel,
                                date=item.date,
                                guid=item.guid,
                                message=message,
                                message_state_new_name=item.message_state_new_name,
                                type='uploaded_file',
                                user__color=item.user__color,
                                user__short_name=self.user.get_short_name,
                                user_id=self.user.id,
                            )
                            logging.debug(f'send_message: {message}')
                            WebSocket.send_message(host=self.host, port=self.port, channel=self.channel, message=message, logger=logger)

                        with transaction.atomic():
                            from tracker.models.messages_files import Messages_files
                            messages_file = Messages_files.objects.create(
                                attfile=fileObj,
                                file_store=self.get_path(fileObj.name),
                                format=item.file_format,
                                mime_type=item.file_mime_type,
                                real_name=item.real_file_name,
                                size=item.file_size,
                            )

                            setattr(item, 'messages_file', messages_file)
                            rec_message(item)

                            logger.debug(f'Запись: {item.real_file_name} ({fileObj.name}) завершена.')

                            self.remove(item.full_path)
                            logger.debug(f'Удаление: {item.full_path} завершено.')

            self.response = dict(status=RPCResponseConstant.statusSuccess)

        except Exception as e:
            raise e



import random
from fabric.api import env
from fabric.contrib.files import exists, sed, append
from fabric.operations import run, local, sudo


class Install:
    def install_python(self, version):
        sudo("apt install nano mc -y")
        sudo("apt install build-essential -y")
        sudo("apt install libreadline-gplv2-dev libncursesw5-dev libssl-dev libsqlite3-dev tk-dev libgdbm-dev libc6-dev libbz2-dev -y")
        sudo(f"wget https://www.python.org/ftp/python/{version}/Python-{version}.tar.xz &&  tar xvf Python-{version}.tar.xz && cd Python-{version}/ && ./configure --enable-shared && make && make install")

    def install_xerces(self, version):
        sudo(f"wget http://mirror.linux-ia64.org/apache//xerces/c/3/sources/xerces-c-{version}.zip && unzip xerces-c-{version}.zip && cd xerces-c-{version}/ && ./configure && make && make install")


class Deploy(Install):
    def getAnySources(self):
        ...

    def __init__(self, REPO_URL, REPO_BRANCH, PYTHON_VERSION="3.7.2", XERCES_VERSION="3.2.2", remake_cpp=True, project_dir_name="project", nanage_module_name="manage.py", WORKERS ="3"):
        self.REPO_URL = REPO_URL
        self.REPO_BRANCH = REPO_BRANCH
        self.PYTHON_VERSION = PYTHON_VERSION
        self.XERCES_VERSION = XERCES_VERSION
        self.remake_cpp = remake_cpp
        self.project_dir_name = project_dir_name
        self.nanage_module_name = nanage_module_name
        self.WORKERS =WORKERS

    def getSiteName(self, port_servise=80):
        return f"{env.host}.{port_servise}"

    def pre_deploy(self):
        if not exists(f"/home/Python-{self.PYTHON_VERSION}"):
            self.install_python(self.PYTHON_VERSION)
            sudo("apt install python3-pip")
            sudo("apt install libtool m4 automake -y")
            sudo("apt install python3-sphinx -y")
            self.install_xerces(self.XERCES_VERSION)

            sudo("apt remove apache2 -y")
            sudo("apt purge apache2 -y")
            sudo("apt autoremove -y")
            sudo("apt purge apache2.* -y")
            sudo("apt install nginx -y")
            sudo("apt install curl -y")
            sudo("apt install python-software-properties -y")
            sudo("curl -sL https://deb.nodesource.com/setup_10.x | sudo -E bash -")
            sudo("apt install nodejs -y")
            sudo("npm install -g grunt-cli -y")
            sudo("ln -s /usr/bin/nodejs /usr/bin/node")

    def deploy(self, port_servise=80):
        self.siteName = self.getSiteName(port_servise)
        self.siteFolder = f"/home/{env.user}/sites/{self.siteName}"
        run(f"rm -rf {self.siteFolder}")
        self.sourceFolder = f"{self.siteFolder}/source"

        sudo("apt update")
        sudo("apt dist-upgrade -y")

        self.deployProcs()
        self.serviceProcs(port=port_servise)

    def deployProcs(self):
        self.createDirectoryStructure()
        self.getSources(sourceFolder=self.sourceFolder, REPO_BRANCH=self.REPO_BRANCH, REPO_URL=self.REPO_URL)
        self.getAnySources()
        run(f"/usr/local/bin/pip3 install --user --upgrade pip")
        run(f"/usr/local/bin/pip3 install --user --no-cache-dir -r {self.sourceFolder}/requirements.txt")
        self.updateSetting()
        self.updateDatabase()
        self.nmpUpdate()
        self.updateStatic()
        if self.remake_cpp:
            self.makeSources()

    def createDirectoryStructure(self):
        for subfolder in ("source", "static", "database"):
            run(f"mkdir -p {self.siteFolder}/{subfolder}")

    def getSources(self, sourceFolder, REPO_BRANCH, REPO_URL):
        if exists(f"{sourceFolder}/.git"):
            run(f"cd {sourceFolder} && git fetch")
        else:
            run(f"git clone --single-branch -b {REPO_BRANCH} {REPO_URL} {sourceFolder}")

        current_commit = local(f"git log -n 1 --format=%H", capture=True)
        run(f"cd {sourceFolder} git reset --hard {current_commit}")

    def updateSetting(self):
        settingPath = f"{self.sourceFolder}/{self.project_dir_name}/settings.py"
        if not exists(settingPath):
            raise Exception(f"File: {settingPath} not exists.")
        sed(settingPath, "DEBUG = True", "DEBUG = False")
        sed(settingPath, "ALLOWED_HOSTS =.+$", f'ALLOWED_HOSTS = ["{env.host}"]')

        secretKeyFile = f"{self.sourceFolder}/{self.project_dir_name}/secret_key.py"
        if not exists(secretKeyFile):
            chars = "abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)"
            key = "".join(random.SystemRandom().choice(chars) for _ in range(50))
            append(secretKeyFile, f'SECRET_KEY="{key}"')
        append(settingPath, '\nfrom .secret_key import SECRET_KEY')

    def updateDatabase(self):
        run(f"cd {self.sourceFolder} && /usr/local/bin/python3.6 {self.nanage_module_name} migrate --noinput")

    def updateStatic(self):
        run(f"cd {self.sourceFolder} && /usr/local/bin/python3.6 {self.nanage_module_name} collectstatic --noinput")

    def nmpUpdate(self):
        if exists(f"{self.sourceFolder}/package.json"):
            run(f"cd {self.sourceFolder} && npm install && grunt")

    def makeSources(self):
        run(f"cd {self.sourceFolder} && make -f Makefile.dist")

    def test(self, port_servise=80):
        self.siteName = self.getSiteName(port_servise)
        self.siteFolder = f"/home/{env.user}/sites/{self.siteName}"
        self.sourceFolder = f"{self.siteFolder}/source"

        run(f"cd {self.sourceFolder} && make -f Makefile.dist check")

    def makeService(self, port_servise=80):
        self.siteName = self.getSiteName(port_servise)
        self.siteFolder = f"/home/{env.user}/sites/{self.siteName}"
        self.sourceFolder = f"{self.siteFolder}/source"

        self.serviceProcs(env.user, port_servise)

    def restartService(self, port_servise=80):
        self.siteName = self.getSiteName(port_servise)
        self._restartService()

    def _restartService(self):
        sudo("systemctl daemon-reload")
        sudo(f"systemctl enable {self.siteName}")
        sudo(f"systemctl restart {self.siteName}")
        sudo(f"systemctl status {self.siteName}")

    def serviceProcs(self, port=80):
        sitesAvailableCfg = f"/etc/nginx/sites-available/{self.siteName}"
        sudo(f"cp /home/{env.user}/.local/lib/python3.6/site-packages/isc_common/deploy/nginx-site-avalabel.conf {sitesAvailableCfg}")
        sed(sitesAvailableCfg, "SITENAME", self.siteName, use_sudo=True)
        sed(sitesAvailableCfg, "SERVERNAME", env.host, use_sudo=True)
        sed(sitesAvailableCfg, "PORT", str(port), use_sudo=True)
        sed(sitesAvailableCfg, "USERNAME", env.user, use_sudo=True)
        if not exists(f"/etc/nginx/sites-enabled/{self.siteName}"):
            sudo(f"ln -s /etc/nginx/sites-available/{self.siteName} /etc/nginx/sites-enabled/{self.siteName}")
        if exists(f"/etc/nginx/sites-enabled/default"):
            sudo("rm /etc/nginx/sites-enabled/default")
        sudo("systemctl reload nginx")
        sudo("systemctl restart nginx")
        servisePath = f"/etc/systemd/system/{self.siteName}.service"
        sudo(f"cp /home/{env.user}/.local/lib/python3.6/site-packages/isc_common/deploy/gunicorn-SITENAME.service {servisePath}")
        sed(servisePath, "SITENAME", self.siteName, use_sudo=True)
        sed(servisePath, "WORKERS", self.WORKERS, use_sudo=True)
        sed(servisePath, "SMTP_SECRET", "Uandrew1965", use_sudo=True)
        sed(servisePath, "SECRET", "mt30718tm", use_sudo=True)
        sed(servisePath, "USERNAME", env.user, use_sudo=True)
        sed(servisePath, "PROJECT_DIR_NAME", self.project_dir_name, use_sudo=True)
        self._restartService()



# Generated by Django 2.1.7 on 2019-03-06 13:34

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.code_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0008_auto_20190221_0934'),
    ]

    operations = [
        migrations.CreateModel(
            name='Params',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', isc_common.fields.code_field.CodeField()),
                ('value', models.TextField(blank=True, null=True)),
                ('user', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Сохраненные папаметры',
            },
        ),
        migrations.AlterUniqueTogether(
            name='params',
            unique_together={('user', 'key')},
        ),
    ]
